@@ -1,7 +1,7 @@
 /**
  * ntfstruncate - Part of the Linux-NTFS project.
  *
- * Copyright (c) 2002-2003 Anton Altaparmakov
+ * Copyright (c) 2002-2005 Anton Altaparmakov
  *
  * This utility will truncate a specified attribute belonging to a
  * specified inode, i.e. file or directory, to a specified length.
@@ -42,14 +42,18 @@
 #ifdef HAVE_ERRNO_H
 #	include <errno.h>
 #endif
+#ifdef HAVE_TIME_H
 #include <time.h>
+#endif
 #ifdef HAVE_GETOPT_H
 #	include <getopt.h>
 #else
 	extern char *optarg;
 	extern int optind;
 #endif
+#ifdef HAVE_LIMITS_H
 #include <limits.h>
+#endif
 #ifndef LLONG_MAX
 #	define LLONG_MAX 9223372036854775807LL
 #endif
@@ -60,8 +64,9 @@
 #include "layout.h"
 #include "volume.h"
 #include "utils.h"
-
-extern const unsigned char attrdef_ntfs12_array[2400];
+#include "attrdef.h"
+#include "version.h"
+#include "logging.h"
 
 const char *EXEC_NAME = "ntfstruncate";
 
@@ -71,7 +76,7 @@ BOOL success = FALSE;
 char *dev_name;
 s64 inode;
 u32 attr_type;
-uchar_t *attr_name = NULL;
+ntfschar *attr_name = NULL;
 u32 attr_name_len;
 s64 new_len;
 
@@ -92,44 +97,11 @@ struct {
 				/* -V, print version and exit. */
 } opts;
 
-/**
- * Dprintf - debugging output (-vv); overriden by quiet (-q)
- */
-static void Dprintf(const char *fmt, ...)
-{
-	va_list ap;
-
-	if (!opts.quiet && opts.verbose > 1) {
-		printf("DEBUG: ");
-		va_start(ap, fmt);
-		vprintf(fmt, ap);
-		va_end(ap);
-	}
-}
-
-/**
- * Eprintf - error output; ignores quiet (-q)
- */
-int Eprintf(const char *fmt, ...)
-{
-	va_list ap;
-
-	fprintf(stderr, "ERROR: ");
-	va_start(ap, fmt);
-	vfprintf(stderr, fmt, ap);
-	va_end(ap);
-	return 0;
-}
-
-/* Generate code for Vprintf() function: Verbose output (-v). */
-GEN_PRINTF(Vprintf, stdout, &opts.verbose, TRUE)
-
-/* Generate code for Qprintf() function: Quietable output (if not -q). */
-GEN_PRINTF(Qprintf, stdout, &opts.quiet,   FALSE)
-
 /**
  * err_exit - error output and terminate; ignores quiet (-q)
  */
+__attribute__((noreturn))
+__attribute__((format(printf, 1, 2)))
 static void err_exit(const char *fmt, ...)
 {
 	va_list ap;
@@ -147,14 +119,14 @@ static void err_exit(const char *fmt, ...)
  */
 static void copyright(void)
 {
-	fprintf(stderr, "Copyright (c) 2002-2003 Anton Altaparmakov\n"
+	fprintf(stderr, "Copyright (c) 2002-2005 Anton Altaparmakov\n"
 			"Copyright (c) 2003 Richard Russon\n"
 			"Truncate a specified attribute of a specified "
 			"inode.\n");
 }
 
 /**
- * license - print licese statement
+ * license - print license statement
  */
 static void license(void)
 {
@@ -164,8 +136,8 @@ static void license(void)
 /**
  * usage - print a list of the parameters to the program
  */
-void usage(void) __attribute__ ((noreturn));
-void usage (void)
+__attribute__((noreturn))
+static void usage(void)
 {
 	copyright();
 	fprintf(stderr, "Usage: %s [options] device inode [attr-type "
@@ -197,7 +169,8 @@ static void parse_options(int argc, char *argv[])
 
 	if (argc && *argv)
 		EXEC_NAME = *argv;
-	fprintf(stderr, "%s v%s\n", EXEC_NAME, VERSION);
+	fprintf(stderr, "%s v%s (libntfs %s)\n", EXEC_NAME, VERSION,
+			ntfs_libntfs_version());
 	while ((c = getopt(argc, argv, "fh?nqvVl")) != EOF)
 		switch (c) {
 		case 'f':
@@ -208,9 +181,11 @@ static void parse_options(int argc, char *argv[])
 			break;
 		case 'q':
 			opts.quiet = 1;
+			ntfs_log_clear_levels(NTFS_LOG_LEVEL_QUIET);
 			break;
 		case 'v':
 			opts.verbose++;
+			ntfs_log_set_levels(NTFS_LOG_LEVEL_VERBOSE);
 			break;
 		case 'V':
 			/* Version number already printed, so just exit. */
@@ -227,9 +202,13 @@ static void parse_options(int argc, char *argv[])
 	if (optind == argc)
 		usage();
 
+	if (opts.verbose > 1)
+		ntfs_log_set_levels(NTFS_LOG_LEVEL_DEBUG | NTFS_LOG_LEVEL_TRACE |
+			NTFS_LOG_LEVEL_VERBOSE | NTFS_LOG_LEVEL_QUIET);
+
 	/* Get the device. */
 	dev_name = argv[optind++];
-	Dprintf("device name = %s\n", dev_name);
+	ntfs_log_verbose("device name = %s\n", dev_name);
 
 	if (optind == argc)
 		usage();
@@ -239,7 +218,7 @@ static void parse_options(int argc, char *argv[])
 	if (*s || !ll || (ll >= LLONG_MAX && errno == ERANGE))
 		err_exit("Invalid inode number: %s\n", argv[optind - 1]);
 	inode = ll;
-	Dprintf("inode = %lli\n", (long long)inode);
+	ntfs_log_verbose("inode = %lli\n", (long long)inode);
 
 	if (optind == argc)
 		usage();
@@ -264,7 +243,7 @@ static void parse_options(int argc, char *argv[])
 		if (optind != argc) {
 			/* Convert the string to little endian Unicode. */
 			attr_name_len = ntfs_mbstoucs(s, &attr_name, 0);
-			if (attr_name_len < 0)
+			if ((int)attr_name_len < 0)
 				err_exit("Invalid attribute name \"%s\": %s\n",
 						s, strerror(errno));
 
@@ -279,19 +258,20 @@ static void parse_options(int argc, char *argv[])
 			attr_name_len = 0;
 		}
 	}
-	Dprintf("attribute type = 0x%x\n", attr_type);
+	ntfs_log_verbose("attribute type = 0x%x\n", (unsigned int)attr_type);
 	if (attr_name == AT_UNNAMED)
-		Dprintf("attribute name = \"\" (UNNAMED)\n");
+		ntfs_log_verbose("attribute name = \"\" (UNNAMED)\n");
 	else
-		Dprintf("attribute name = \"%s\" (length %i Unicode "
-				"characters)\n", s2, attr_name_len);
+		ntfs_log_verbose("attribute name = \"%s\" (length %u Unicode "
+				"characters)\n", s2,
+				(unsigned int)attr_name_len);
 
 	/* Get the new length. */
 	ll = strtoll(s, &s2, 0);
 	if (*s2 || ll < 0 || (ll >= LLONG_MAX && errno == ERANGE))
 		err_exit("Invalid new length: %s\n", s);
 	new_len = ll;
-	Dprintf("new length = %lli\n", new_len);
+	ntfs_log_verbose("new length = %lli\n", new_len);
 }
 
 /**
@@ -304,9 +284,9 @@ static void parse_options(int argc, char *argv[])
  * terminating null byte. If a unicode character was encountered which could
  * not be converted -1 is returned.
  */
-static int ucstos(char *dest, const uchar_t *src, int maxlen)
+static int ucstos(char *dest, const ntfschar *src, int maxlen)
 {
-	uchar_t u;
+	ntfschar u;
 	int i;
 
 	/* Need one byte for null terminator. */
@@ -358,13 +338,13 @@ static void dump_resident_attr_val(ATTR_TYPES type, char *val, u32 val_len)
 		printf("%s\n", todo);
 		return;
 	case AT_VOLUME_NAME:
-		printf("Volume name length = %i\n", val_len);
+		printf("Volume name length = %u\n", (unsigned int)val_len);
 		if (val_len) {
 			buf = calloc(1, val_len);
 			if (!buf)
 				err_exit("Failed to allocate internal buffer: "
 						"%s\n", strerror(errno));
-			i = ucstos(buf, (uchar_t*)val, val_len);
+			i = ucstos(buf, (ntfschar*)val, val_len);
 			if (i == -1)
 				printf("Volume name contains non-displayable "
 						"Unicode characters.\n");
@@ -467,7 +447,7 @@ static void dump_resident_attr_val(ATTR_TYPES type, char *val, u32 val_len)
 		printf("Cannot display unknown %s defined attribute type 0x%x"
 				".\n", u >=
 				le32_to_cpu(AT_FIRST_USER_DEFINED_ATTRIBUTE) ?
-				"user" : "system", u);
+				"user" : "system", (unsigned int)u);
 	}
 }
 
@@ -497,7 +477,8 @@ static void dump_resident_attr(ATTR_RECORD *a)
 /**
  * dump_mapping_pairs_array
  */
-static void dump_mapping_pairs_array(char *b, unsigned int max_len)
+static void dump_mapping_pairs_array(char *b __attribute__((unused)),
+	unsigned int max_len __attribute__((unused)))
 {
 	// TODO
 	return;
@@ -554,7 +535,8 @@ static void dump_attr_record(MFT_RECORD *m, ATTR_RECORD *a)
 	printf("-- Beginning dump of attribute record at offset 0x%x. --\n",
 			(unsigned)((u8*)a - (u8*)m));
 	if (a->type == AT_END) {
-		printf("Attribute type = 0x%x ($END)\n", le32_to_cpu(AT_END));
+		printf("Attribute type = 0x%x ($END)\n",
+				(unsigned int)le32_to_cpu(AT_END));
 		u = le32_to_cpu(a->length);
 		printf("Length of resident part = %u (0x%x)\n", u, u);
 		return;
@@ -569,7 +551,7 @@ static void dump_attr_record(MFT_RECORD *m, ATTR_RECORD *a)
 //		printf("name = %c%c%c%c%c\n", *p, p[1], p[2], p[3], p[4]);
 //		}
 		if (ucstos(s, attr_defs[i].name, sizeof(s)) == -1) {
-			Eprintf("Could not convert Unicode string to single "
+			ntfs_log_error("Could not convert Unicode string to single "
 				"byte string in current locale.\n");
 			strncpy(s, "Error converting Unicode string",
 					sizeof(s));
@@ -585,11 +567,11 @@ static void dump_attr_record(MFT_RECORD *m, ATTR_RECORD *a)
 			cpu_to_le16(a->name_offset));
 	u = a->flags;
 	if (a->name_length) {
-		if (ucstos(s, (uchar_t*)((char*)a +
+		if (ucstos(s, (ntfschar*)((char*)a +
 				cpu_to_le16(a->name_offset)),
 				min((int)sizeof(s),
 						a->name_length + 1)) == -1) {
-			Eprintf("Could not convert Unicode string to single "
+			ntfs_log_error("Could not convert Unicode string to single "
 				"byte string in current locale.\n");
 			strncpy(s, "Error converting Unicode string",
 					sizeof(s));
@@ -706,17 +688,15 @@ static void ntfstruncate_exit(void)
 		ntfs_attr_close(na);
 	/* Close the inode. */
 	if (ni && ntfs_inode_close(ni)) {
-		fprintf(stderr, "Warning: Failed to close inode %lli: %s\n",
-				(long long)inode, strerror(errno));
+		ntfs_log_perror("Warning: Failed to close inode %lli",
+				(long long)inode);
 	}
 	/* Unmount the volume. */
 	err = ntfs_umount(vol, 0);
 	if (err == -1)
-		fprintf(stderr, "Warning: Could not umount %s: %s\n", dev_name,
-				strerror(errno));
+		ntfs_log_perror("Warning: Could not umount %s", dev_name);
 	/* Free the attribute name if it exists. */
-	if (attr_name && attr_name != AT_UNNAMED)
-		free(attr_name);
+	ntfs_ucsfree(attr_name);
 }
 
 /**
@@ -727,6 +707,8 @@ int main(int argc, char **argv)
 	unsigned long mnt_flags, ul;
 	int err;
 
+	ntfs_log_set_handler(ntfs_log_handler_outerr);
+
 	/* Initialize opts to zero / required values. */
 	memset(&opts, 0, sizeof(opts));
 
@@ -743,10 +725,10 @@ int main(int argc, char **argv)
 
 	/* Make sure the file system is not mounted. */
 	if (ntfs_check_if_mounted(dev_name, &mnt_flags))
-		Eprintf("Failed to determine whether %s is mounted: %s\n",
-				dev_name, strerror(errno));
+		ntfs_log_perror("Failed to determine whether %s is mounted",
+				dev_name);
 	else if (mnt_flags & NTFS_MF_MOUNTED) {
-		Eprintf("%s is mounted.\n", dev_name);
+		ntfs_log_error("%s is mounted.\n", dev_name);
 		if (!opts.force)
 			err_exit("Refusing to run!\n");
 		fprintf(stderr, "ntfstruncate forced anyway. Hope /etc/mtab "
@@ -755,7 +737,7 @@ int main(int argc, char **argv)
 
 	/* Mount the device. */
 	if (opts.no_action) {
-		Qprintf("Running in READ-ONLY mode!\n");
+		ntfs_log_quiet("Running in READ-ONLY mode!\n");
 		ul = MS_RDONLY;
 	} else
 		ul = 0;
@@ -766,7 +748,7 @@ int main(int argc, char **argv)
 	/* Register our exit function which will unlock and close the device. */
 	err = atexit(&ntfstruncate_exit);
 	if (err == -1) {
-		Eprintf("Could not set up exit() function because atexit() "
+		ntfs_log_error("Could not set up exit() function because atexit() "
 				"failed: %s Aborting...\n", strerror(errno));
 		ntfstruncate_exit();
 		exit(1);
@@ -781,11 +763,11 @@ int main(int argc, char **argv)
 	/* Open the specified attribute. */
 	na = ntfs_attr_open(ni, attr_type, attr_name, attr_name_len);
 	if (!na)
-		err_exit("Failed to open attribute 0x%x: %s\n", attr_type,
-				strerror(errno));
+		err_exit("Failed to open attribute 0x%x: %s\n",
+				(unsigned int)attr_type, strerror(errno));
 
 	if (!opts.quiet && opts.verbose > 1) {
-		Dprintf("Dumping mft record before calling "
+		ntfs_log_verbose("Dumping mft record before calling "
 				"ntfs_attr_truncate():\n");
 		dump_mft_record(ni->mrec);
 	}
@@ -793,11 +775,11 @@ int main(int argc, char **argv)
 	/* Truncate the attribute. */
 	err = ntfs_attr_truncate(na, new_len);
 	if (err)
-		err_exit("Failed to truncate attribute 0x%x: %s\n", attr_type,
-				strerror(errno));
+		err_exit("Failed to truncate attribute 0x%x: %s\n",
+				(unsigned int)attr_type, strerror(errno));
 
 	if (!opts.quiet && opts.verbose > 1) {
-		Dprintf("Dumping mft record after calling "
+		ntfs_log_verbose("Dumping mft record after calling "
 				"ntfs_attr_truncate():\n");
 		dump_mft_record(ni->mrec);
 	}
@@ -815,17 +797,15 @@ int main(int argc, char **argv)
 	/* Unmount the volume. */
 	err = ntfs_umount(vol, 0);
 	if (err == -1)
-		fprintf(stderr, "Warning: Failed to umount %s: %s\n", dev_name,
-				strerror(errno));
+		ntfs_log_perror("Warning: Failed to umount %s", dev_name);
 
 	/* Free the attribute name if it exists. */
-	if (attr_name && attr_name != AT_UNNAMED)
-		free(attr_name);
+	ntfs_ucsfree(attr_name);
 
 	/* Finally, disable our ntfstruncate_exit() handler. */
 	success = TRUE;
 
-	Qprintf("ntfstruncate completed successfully. Have a nice day.\n");
+	ntfs_log_quiet("ntfstruncate completed successfully. Have a nice day.\n");
 	return 0;
 }
 
